@@ -13,15 +13,15 @@ App::Ack - A container for functions for the ack program
 
 =head1 VERSION
 
-Version 1.89_02
+Version 1.93_01
 
 =cut
 
 our $VERSION;
 our $COPYRIGHT;
 BEGIN {
-    $VERSION = '1.89_02';
-    $COPYRIGHT = 'Copyright 2005-2009 Andy Lester, all rights reserved.';
+    $VERSION = '1.93_01';
+    $COPYRIGHT = 'Copyright 2005-2010 Andy Lester.';
 }
 
 our $fh;
@@ -72,11 +72,13 @@ BEGIN {
 
     %mappings = (
         actionscript => [qw( as mxml )],
+        ada         => [qw( ada adb ads )],
         asm         => [qw( asm s )],
         batch       => [qw( bat cmd )],
         binary      => q{Binary files, as defined by Perl's -B op (default: off)},
         cc          => [qw( c h xs )],
         cfmx        => [qw( cfc cfm cfml )],
+        clojure     => [qw( clj )],
         cpp         => [qw( cpp cc cxx m hpp hh h hxx )],
         csharp      => [qw( cs )],
         css         => [qw( css )],
@@ -91,18 +93,19 @@ BEGIN {
         jsp         => [qw( jsp jspx jhtm jhtml )],
         lisp        => [qw( lisp lsp )],
         lua         => [qw( lua )],
-        make        => q{Makefiles},
+        make        => q{Makefiles (including *.mk and *.mak)},
         mason       => [qw( mas mhtml mpl mtxt )],
         objc        => [qw( m h )],
         objcpp      => [qw( mm h )],
         ocaml       => [qw( ml mli )],
         parrot      => [qw( pir pasm pmc ops pod pg tg )],
         perl        => [qw( pl pm pod t )],
-        php         => [qw( php phpt php3 php4 php5 )],
+        php         => [qw( php phpt php3 php4 php5 phtml)],
         plone       => [qw( pt cpt metadata cpy py )],
         python      => [qw( py )],
         rake        => q{Rakefiles},
-        ruby        => [qw( rb rhtml rjs rxml erb rake )],
+        ruby        => [qw( rb rhtml rjs rxml erb rake spec )],
+        scala       => [qw( scala )],
         scheme      => [qw( scm ss )],
         shell       => [qw( sh bash csh tcsh ksh zsh )],
         skipped     => q{Files, but not directories, normally skipped by ack (default: off)},
@@ -113,6 +116,8 @@ BEGIN {
         text        => q{Text files, as defined by Perl's -T op (default: off)},
         tt          => [qw( tt tt2 ttml )],
         vb          => [qw( bas cls frm ctl vb resx )],
+        verilog     => [qw( v vh sv )],
+        vhdl        => [qw( vhd vhdl )],
         vim         => [qw( vim )],
         yaml        => [qw( yaml yml )],
         xml         => [qw( xml dtd xslt ent )],
@@ -125,6 +130,8 @@ BEGIN {
             }
         }
     }
+    # add manually Makefile extensions
+    push @{$types{$_}}, 'make' for qw{ mk mak };
 
     # These have to be checked before any filehandle diddling.
     $output_to_pipe  = not -t *STDOUT;
@@ -167,6 +174,12 @@ sub read_ackrc {
             chomp @lines;
             close $fh or App::Ack::die( "$filename: $!\n" );
 
+            # get rid of leading and trailing whitespaces
+            for ( @lines ) {
+               s/^\s+//;
+               s/\s+$//;
+            }
+
             return @lines;
         }
     }
@@ -196,6 +209,7 @@ sub get_command_line_options {
         'color|colour!'         => \$opt{color},
         'color-match=s'         => \$ENV{ACK_COLOR_MATCH},
         'color-filename=s'      => \$ENV{ACK_COLOR_FILENAME},
+        'color-lineno=s'        => \$ENV{ACK_COLOR_LINENO},
         'column!'               => \$opt{column},
         count                   => \$opt{count},
         'env!'                  => sub { }, # ignore this option, it is handled beforehand
@@ -211,10 +225,10 @@ sub get_command_line_options {
         'i|ignore-case'         => \$opt{i},
         'lines=s'               => sub { shift; my $val = shift; push @{$opt{lines}}, $val },
         'l|files-with-matches'  => \$opt{l},
-        'L|files-without-match' => sub { $opt{l} = $opt{v} = 1 },
+        'L|files-without-matches' => sub { $opt{l} = $opt{v} = 1 },
         'm|max-count=i'         => \$opt{m},
         'match=s'               => \$opt{regex},
-        n                       => \$opt{n},
+        'n|no-recurse'          => \$opt{n},
         o                       => sub { $opt{output} = '$&' },
         'output=s'              => \$opt{output},
         'pager=s'               => \$opt{pager},
@@ -222,6 +236,8 @@ sub get_command_line_options {
         'passthru'              => \$opt{passthru},
         'print0'                => \$opt{print0},
         'Q|literal'             => \$opt{Q},
+        'r|R|recurse'           => sub { $opt{n} = 0 },
+        'show-types'            => \$opt{show_types},
         'smart-case!'           => \$opt{smart_case},
         'sort-files'            => \$opt{sort_files},
         'u|unrestricted'        => \$opt{u},
@@ -266,7 +282,7 @@ sub get_command_line_options {
     my $parser = Getopt::Long::Parser->new();
     $parser->configure( 'bundling', 'no_ignore_case', );
     $parser->getoptions( %{$getopt_specs} ) or
-        App::Ack::die( 'See ack --help or ack --man for options.' );
+        App::Ack::die( 'See ack --help, ack --help-types or ack --man for options.' );
 
     my $to_screen = not output_to_pipe();
     my %defaults = (
@@ -468,13 +484,13 @@ use constant TEXT => 'text';
 sub filetypes {
     my $filename = shift;
 
-    return 'skipped' unless is_searchable( $filename );
-
     my $basename = $filename;
     $basename =~ s{.*[$dir_sep_chars]}{};
 
+    return 'skipped' unless is_searchable( $basename );
+
     my $lc_basename = lc $basename;
-    return ('make',TEXT)        if $lc_basename eq 'makefile';
+    return ('make',TEXT)        if $lc_basename eq 'makefile' || $lc_basename eq 'gnumakefile'; 
     return ('rake','ruby',TEXT) if $lc_basename eq 'rakefile';
 
     # If there's an extension, look it up
@@ -533,6 +549,8 @@ Recognized files:
   /[._].*\.swp$/  - Vi(m) swap files
   /core\.\d+$/    - core dumps
 
+Note that I<$filename> must be just a file, not a full path.
+
 =cut
 
 sub is_searchable {
@@ -541,7 +559,9 @@ sub is_searchable {
     # If these are updated, update the --help message
     return if $filename =~ /[.]bak$/;
     return if $filename =~ /~$/;
-    return if $filename =~ m{[$dir_sep_chars]?(?:#.+#|core\.\d+|[._].*\.swp)$}o;
+    return if $filename =~ m{^#.*#$}o;
+    return if $filename =~ m{^core\.\d+$}o;
+    return if $filename =~ m{[._].*\.swp$}o;
 
     return 1;
 }
@@ -550,12 +570,16 @@ sub is_searchable {
 
 Returns a regex object based on a string and command-line options.
 
+Dies when the regex $str is undefinied (i.e. not given on command line).
+
 =cut
 
 sub build_regex {
     my $str = shift;
     my $opt = shift;
 
+    defined $str or App::Ack::die( 'No regular expression found.' );
+
     $str = quotemeta( $str ) if $opt->{Q};
     if ( $opt->{w} ) {
         $str = "\\b$str" if $str =~ /^\w/;
@@ -691,8 +715,8 @@ Search output:
   --line=NUM            Only print line(s) NUM of each file
   -l, --files-with-matches
                         Only print filenames containing matches
-  -L, --files-without-match
-                        Only print filenames with no match
+  -L, --files-without-matches
+                        Only print filenames with no matches
   -o                    Show only the part of a line matching PATTERN
                         (turns off text highlighting)
   --passthru            Print all lines, whether matching or not
@@ -719,8 +743,8 @@ Search output:
                         only works with -f, -g, -l, -L or -c.
 
 File presentation:
-  --pager=COMMAND       Pipes all ack output through COMMAND.
-                        Ignored if output is redirected.
+  --pager=COMMAND       Pipes all ack output through COMMAND.  For example,
+                        --pager="less -R".  Ignored if output is redirected.
   --nopager             Do not send output through a pager.  Cancels any
                         setting in ~/.ackrc, ACK_PAGER or ACK_PAGER_COLOR.
   --[no]heading         Print a filename heading above each file's results.
@@ -733,7 +757,8 @@ File presentation:
                         output is redirected, or on Windows)
   --[no]colour          Same as --[no]color
   --color-filename=COLOR
-  --color-match=COLOR   Set the color for matches and filenames.
+  --color-match=COLOR
+  --color-lineno=COLOR  Set the color for filenames, matches, and line numbers.
   --flush               Flush output immediately, even when ack is used
                         non-interactively (when output goes to a pipe or
                         file).
@@ -743,13 +768,15 @@ File finding:
                         The PATTERN must not be specified.
   -g REGEX              Same as -f, but only print files matching REGEX.
   --sort-files          Sort the found files lexically.
+  --show-types          Show which types each file has.
 
 File inclusion/exclusion:
   -a, --all-types       All file types searched;
                         Ignores CVS, .svn and other ignored directories
   -u, --unrestricted    All files and directories searched
   --[no]ignore-dir=name Add/Remove directory from the list of ignored dirs
-  -n                    No descending into subdirectories
+  -r, -R, --recurse     Recurse into subdirectories (ack's default behavior)
+  -n, --no-recurse      No descending into subdirectories
   -G REGEX              Only search files that match REGEX
 
   --perl                Include only Perl files.
@@ -859,10 +886,11 @@ sub get_version_statement {
         my $ext = $Config::Config{_exe};
         $this_perl .= $ext unless $this_perl =~ m/$ext$/i;
     }
+    my $ver = sprintf( '%vd', $^V );
 
     return <<"END_OF_VERSION";
 ack $VERSION
-Running under Perl $] at $this_perl
+Running under Perl $ver at $this_perl
 
 $copyright
 
@@ -905,6 +933,7 @@ sub load_colors {
 
     $ENV{ACK_COLOR_MATCH}    ||= 'black on_yellow';
     $ENV{ACK_COLOR_FILENAME} ||= 'bold green';
+    $ENV{ACK_COLOR_LINENO}   ||= 'bold yellow';
 
     return;
 }
@@ -951,17 +980,27 @@ sub print_count {
     my $nmatches = shift;
     my $ors = shift;
     my $count = shift;
+    my $show_filename = shift;
 
-    App::Ack::print( $filename );
-    App::Ack::print( ':', $nmatches ) if $count;
+    if ($show_filename) {
+        App::Ack::print( $filename );
+        App::Ack::print( ':', $nmatches ) if $count;
+    } else {
+        App::Ack::print( $nmatches ) if $count;
+    }
     App::Ack::print( $ors );
 }
 
 sub print_count0 {
     my $filename = shift;
     my $ors = shift;
+    my $show_filename = shift;
 
-    App::Ack::print( $filename, ':0', $ors );
+    if ($show_filename) {
+        App::Ack::print( $filename, ':0', $ors );
+    } else {
+        App::Ack::print( '0', $ors );
+    }
 }
 
 
@@ -1034,7 +1073,7 @@ sub search_resource {
 
             if ( $keep_context ) {
                 if ( $after ) {
-                    print_match_or_context( $opt, 0, $., $_ );
+                    print_match_or_context( $opt, 0, $., $-[0], $+[0], $_ );
                     $after--;
                 }
                 elsif ( $before_context ) {
@@ -1069,7 +1108,7 @@ sub search_resource {
         }
         if ( $keep_context ) {
             if ( @before ) {
-                print_match_or_context( $opt, 0, $before_starts_at_line, @before );
+                print_match_or_context( $opt, 0, $before_starts_at_line, $-[0], $+[0], @before );
                 @before = ();
                 $before_starts_at_line = 0;
             }
@@ -1080,7 +1119,7 @@ sub search_resource {
                 $after = $after_context;
             }
         }
-        print_match_or_context( $opt, 1, $., $_ );
+        print_match_or_context( $opt, 1, $., $-[0], $+[0], $_ );
 
         last if $max && ( $nmatches >= $max ) && !$after;
     } # while
@@ -1089,16 +1128,18 @@ sub search_resource {
 }   # search_resource()
 
 
-=head2 print_match_or_context( $opt, $is_match, $starting_line_no, @lines )
+=head2 print_match_or_context( $opt, $is_match, $starting_line_no, $match_start, $match_end, @lines )
 
 Prints out a matching line or a line of context around a match.
 
 =cut
 
 sub print_match_or_context {
-    my $opt      = shift; # opts array
-    my $is_match = shift; # is there a match on the line?
-    my $line_no  = shift;
+    my $opt         = shift; # opts array
+    my $is_match    = shift; # is there a match on the line?
+    my $line_no     = shift;
+    my $match_start = shift;
+    my $match_end   = shift;
 
     my $color         = $opt->{color};
     my $heading       = $opt->{heading};
@@ -1132,7 +1173,11 @@ sub print_match_or_context {
 
         if ( $show_filename ) {
             App::Ack::print_filename($display_filename, $sep) if not $heading;
-            App::Ack::print_line_no($line_no, $sep);
+            my $display_line_no =
+                $color
+                    ? Term::ANSIColor::colored( $line_no, $ENV{ACK_COLOR_LINENO} )
+                    : $line_no;
+            App::Ack::print_line_no($display_line_no, $sep);
         }
 
         if ( $output_func ) {
@@ -1141,8 +1186,6 @@ sub print_match_or_context {
             }
         }
         else {
-            my $col = $-[0] + 1;
-
             if ( $color && $is_match && $regex &&
                  s/$regex/Term::ANSIColor::colored( substr($_, $-[0], $+[0] - $-[0]), $ENV{ACK_COLOR_MATCH} )/eg ) {
                 # At the end of the line reset the color and remove newline
@@ -1153,7 +1196,7 @@ sub print_match_or_context {
                 s/[\r\n]*\z//;
             }
             if ( $show_column ) {
-                App::Ack::print_column_no( $col, $sep );
+                App::Ack::print_column_no( $match_start+1, $sep );
             }
             App::Ack::print($_ . "\n");
         }
@@ -1168,7 +1211,18 @@ sub print_match_or_context {
 } # scope around search_resource() and print_match_or_context()
 
 
-=head2 search_and_list( $fh, $filename, \%opt )
+TOTAL_COUNT_SCOPE: {
+my $total_count;
+
+sub get_total_count {
+    return $total_count;
+}
+
+sub reset_total_count {
+    $total_count = 0;
+}
+
+=head2 search_and_list( $res, \%opt )
 
 Optimized version of searching for -l and --count, which do not
 show lines.
@@ -1182,6 +1236,7 @@ sub search_and_list {
     my $nmatches = 0;
     my $count = $opt->{count};
     my $ors = $opt->{print0} ? "\0" : "\n"; # output record separator
+    my $show_filename = $opt->{show_filename};
 
     my $regex = qr/$opt->{regex}/;
 
@@ -1204,16 +1259,22 @@ sub search_and_list {
         }
     }
 
-    if ( $nmatches ) {
-        App::Ack::print_count( $res->name, $nmatches, $ors, $count );
-    }
-    elsif ( $count && !$opt->{l} ) {
-        App::Ack::print_count0( $res->name, $ors );
+    if ( $opt->{show_total} ) {
+        $total_count += $nmatches;
+    } else {
+        if ( $nmatches ) {
+            App::Ack::print_count( $res->name, $nmatches, $ors, $count, $show_filename );
+        }
+        elsif ( $count && !$opt->{l} ) {
+            App::Ack::print_count0( $res->name, $ors, $show_filename );
+        }
     }
 
     return $nmatches ? 1 : 0;
 }   # search_and_list()
 
+} # scope around $total_count
+
 
 =head2 filetypes_supported_set
 
@@ -1242,12 +1303,14 @@ sub print_files {
 
     my $ors = $opt->{print0} ? "\0" : "\n";
 
+    my $nmatches = 0;
     while ( defined ( my $file = $iter->() ) ) {
-        App::Ack::print $file, $ors;
+        App::Ack::print $file, $opt->{show_types} ? " => " . join( ',', filetypes( $file ) ) : (),  $ors;
+        $nmatches++;
         last if $opt->{1};
     }
 
-    return;
+    return $nmatches;
 }
 
 =head2 print_files_with_matches( $iter, $opt )
@@ -1260,6 +1323,17 @@ sub print_files_with_matches {
     my $iter = shift;
     my $opt = shift;
 
+    # if we have -l and only 1 file given on command line (this means
+    # show_filename is set to 0), we want to see the filename nevertheless
+    $opt->{show_filename} = 1 if $opt->{l};
+
+    $opt->{show_filename} = 0 if $opt->{h};
+    $opt->{show_filename} = 1 if $opt->{H};
+
+    # abuse options to hand in the show_total parameter to search_and_list
+    $opt->{show_total} = $opt->{count} && !$opt->{show_filename};
+    reset_total_count();
+
     my $nmatches = 0;
     while ( defined ( my $filename = $iter->() ) ) {
         my $repo = App::Ack::Repository::Basic->new( $filename );
@@ -1272,6 +1346,10 @@ sub print_files_with_matches {
         $repo->close();
     }
 
+    if ( $nmatches && $opt->{show_total} ) {
+        App::Ack::print_count('', get_total_count(), "\n", 1, 0  )
+    }
+
     return $nmatches;
 }
 
@@ -1437,14 +1515,14 @@ sub get_iterator {
     if ( $g_regex ) {
         $file_filter
             = $opt->{u}   ? sub { $File::Next::name =~ /$g_regex/ } # XXX Maybe this should be a 1, no?
-            : $opt->{all} ? sub { $starting_point{ $File::Next::name } || ( $File::Next::name =~ /$g_regex/ && is_searchable( $File::Next::name ) ) }
+            : $opt->{all} ? sub { $starting_point{ $File::Next::name } || ( $File::Next::name =~ /$g_regex/ && is_searchable( $_ ) ) }
             :               sub { $starting_point{ $File::Next::name } || ( $File::Next::name =~ /$g_regex/ && is_interesting( @_ ) ) }
             ;
     }
     else {
         $file_filter
             = $opt->{u}   ? sub {1}
-            : $opt->{all} ? sub { $starting_point{ $File::Next::name } || is_searchable( $File::Next::name ) }
+            : $opt->{all} ? sub { $starting_point{ $File::Next::name } || is_searchable( $_ ) }
             :               sub { $starting_point{ $File::Next::name } || is_interesting( @_ ) }
             ;
     }
@@ -1469,7 +1547,7 @@ sub get_iterator {
 sub set_up_pager {
     my $command = shift;
 
-    return unless App::Ack::output_to_pipe();
+    return if App::Ack::output_to_pipe();
 
     my $pager;
     if ( not open( $pager, '|-', $command ) ) {
@@ -1501,23 +1579,29 @@ sub output_to_pipe {
     return $output_to_pipe;
 }
 
+=head2 exit_from_ack
+
+Exit from the application with the correct exit code.
+
+Returns with 0 if a match was found, otherwise with 1. The number of matches is
+handed in as the only argument.
+
+=cut
+
+sub exit_from_ack {
+    my $nmatches = shift;
+
+    my $rc = $nmatches ? 0 : 1;
+    exit $rc;
+}
+
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005-2009 Andy Lester, all rights reserved.
+Copyright 2005-2010 Andy Lester.
 
 This program is free software; you can redistribute it and/or modify
-it under the terms of either:
-
-=over 4
-
-=item * the GNU General Public License as published by the Free
-Software Foundation; either version 1, or (at your option) any later
-version, or
-
-=item * the "Artistic License" which comes with Perl 5.
-
-=back
+it under the terms of the Artistic License v2.0.
 
 =cut
 
